pub(crate) use deprecated_type_alias::{deprecated_type_alias, NumpyDeprecatedTypeAlias};
pub(crate) use numpy_legacy_random::{numpy_legacy_random, NumpyLegacyRandom};

mod deprecated_type_alias;
mod numpy_legacy_random;
