from numpy import (
	cos,
	int8,
	int16,
	int32,
	int64,
	sin,
	tan,
	uint8,
	uint16,
	uint32,
	uint64,
)

if True:
	# inside nested block
	from numpy import (
		cos,
		int8,
		int16,
		int32,
		int64,
		sin,
		tan,
		uint8,
		uint16,
		uint32,
		uint64,
	)
