x = 1  # pyright: ignore
x = 1  # pyright:ignore
x = 1  # pyright: ignore[attr-defined]  # pyright: ignore

x = 1
x = 1  # pyright ignore
x = 1  # pyright ignore  # noqa
x = 1  # pyright: ignore[attr-defined]
x = 1  # pyright: ignore[attr-defined, name-defined]
x = 1  # pyright: ignore[attr-defined]  # pyright: ignore[type-mismatch]
x = 1  # pyright: ignore[type-mismatch]  # noqa
x = 1  # pyright: ignore [attr-defined]
x = 1  # pyright: ignore [attr-defined, name-defined]
x = 1  # pyright: ignore [type-mismatch]  # noqa
x = 1  # pyright: Union[int, str]
x = 1  # pyright: ignoreme
