class Foo:
    def __str__(self):
        ...

    def __repr__(self):
        ...

    def __len__(self):
        ...

    def __length_hint__(self):
        ...

    def __init__(self):
        ...

    def __del__(self):
        ...

    def __bool__(self):
        ...

    def __bytes__(self):
        ...

    def __format__(self, format_spec):
        ...

    def __contains__(self, item):
        ...

    def __complex__(self):
        ...

    def __int__(self):
        ...

    def __float__(self):
        ...

    def __index__(self):
        ...
