# E2502
print("שלום‬")

# E2502
example = "x‏" * 100  #    "‏x" is assigned

# E2502
if access_level != "none‮⁦":  # Check if admin ⁩⁦' and access_level != 'user
    print("You are an admin.")


# E2502
def subtract_funds(account: str, amount: int):
    """Subtract funds from bank account then ⁧"""
    return
    bank[account] -= amount
    return


# OK
print("\u202B\u202E\u05e9\u05DC\u05D5\u05DD\u202C")


# OK
print("\N{RIGHT-TO-LEFT MARK}")

# OK
print("Hello World")
