def f():
    # isort: off
    import sys
    import os
    import collections
    # isort: on


def f():
    # ruff: isort: off
    import sys
    import os
    import collections
    # ruff: isort: on


def f():
    import sys
    import os  # isort: skip
    import collections
    import abc


def f():
    import sys
    import os  # isort:skip
    import collections
    import abc
