import numpy as npy
import numpy as np
import numpy

# Error
npy.bool
npy.int

if dtype == np.object:
    ...

result = result.select_dtypes([np.byte, np.ubyte, np.short, np.ushort, np.int, np.long])

pdf = pd.DataFrame(
    data=[[1, 2, 3]],
    columns=["a", "b", "c"],
    dtype=numpy.object,
)

_ = arr.astype(np.int)
