from foo import (  # Comment on `foo`
    Member as Alias,  # Comment on `Alias`
)

from bar import (  # Comment on `bar`
    Member,  # Comment on `Member`
)

from baz import (  # Comment on `baz`
    Member as Alias  # Comment on `Alias`
)

from bop import (  # Comment on `bop`
    Member  # Comment on `Member`
)
