def func_all_spaces():
    # No error
    print("spaces")

def func_tabs():
    # No error
	print("tabs")

def func_mixed_start_with_tab():
    # E101
	  print("mixed starts with tab")

def func_mixed_start_with_space():
    # E101
 	  	  	   print("mixed starts with space")

def xyz():
    # E101
   	print("xyz");
